﻿using System;
using System.Globalization;

using Platform.Net;

namespace Platform.Geo
{
    public class GeoPosition : GeoShape
    {
        public static GeoPosition Schiphol
        {
            get { return new GeoPosition { Latitude = 52.309, Longitude = 4.7635 }; }
        }

        /// <summary>
        /// Gets or sets the latitude
        /// </summary>
        public double Latitude { get; set; }

        /// <summary>
        /// Gets or sets the longitude
        /// </summary>
        public double Longitude { get; set; }

        /// <summary>
        /// Gets or sets the longitude
        /// </summary>
        public double? Altitude { get; set; }

        /// <summary>
        /// Convert to WKT
        /// </summary>
        /// <returns></returns>
        public override string ToWKT()
        {
            if( Altitude != null )
                return string.Format(CultureInfo.InvariantCulture, "POINT({0} {1} {2})", Longitude, Latitude, Altitude.Value);
            else return string.Format(CultureInfo.InvariantCulture, "POINT({0} {1})", Longitude, Latitude);
        }

        /// <summary>
        /// Parse from WKT
        /// </summary>
        /// <param name="wkt"></param>
        /// <returns></returns>
        public static GeoPosition FromWKT( string wkt )
        {
            // check wkt
            if (string.IsNullOrEmpty(wkt)) return null;

            // read point
            return FromWKT(new NetStringReader(wkt));
        }

        /// <summary>
        /// Parse from WKT
        /// </summary>
        /// <param name="wkt"></param>
        /// <returns></returns>
        public static GeoPosition FromWKT(NetStringReader reader)
        {
            reader.Skip();
            if (reader.ReadPattern(true, "POINT") == -1)
                throw new FormatException("Missing literal 'POINT' in wkt");
            reader.Skip();
            reader.Read('(');
            var result = ReadWktPointData(reader);
            reader.Skip();
            reader.Read(')');
            return result;
        }

        /// <summary>
        /// Get Bounds
        /// </summary>
        /// <returns></returns>
        public override GeoRectangle GetBounds()
        {
            return new GeoRectangle
            {
                NorthwestCorner = this,
                SoutheastCorner = this
            };
        }
    }
}
